# -*- coding: utf-8 -*-

import sys
from ixiaerror import IxiaError
from ixiangpf import IxiaNgpf
from ixiautil import PartialClass, make_hltapi_fail

class IxiaNgpf(PartialClass, IxiaNgpf):
	def emulation_static_macsec_config(self, mode, **kwargs):
		r'''
		#Procedure Header
		 Name:
		    emulation_static_macsec_config
		
		 Description:
		    This procedure will add and configure Static Macsec to a particular Ixia Interface.
		
		 Synopsis:
		    emulation_static_macsec_config
		        -mode                     CHOICES create
		                                  CHOICES delete
		                                  CHOICES modify
		                                  CHOICES enable
		                                  CHOICES disable
		        [-port_handle             REGEXP ^[0-9]+/[0-9]+/[0-9]+$]
		        [-handle                  ANY]
		x       [-enable_encrypted_vlan   ANY]
		x       [-encrypted_traffic_type  CHOICES statefulL47 statelessL23]
		x       [-source_ip               ANY]
		x       [-source_ip_step          ANY]
		x       [-dut_mac                 ANY]
		x       [-dut_mac_step            ANY]
		x       [-send_grat_arp           CHOICES 0 1]
		x       [-fixed_pn                NUMERIC]
		x       [-mv_fixed_xpn            ANY]
		x       [-active_device           ANY]
		x       [-cipher_suite            CHOICES aes128
		x                                 CHOICES aes256
		x                                 CHOICES aesxpn128
		x                                 CHOICES aesxpn256]
		x       [-confidentiality_offset  ANY]
		x       [-enable_encrypted_vlans  ANY]
		x       [-encrypted_vlan_count    NUMERIC]
		x       [-tx_sak_pool_size        NUMERIC]
		x       [-enable_confidentiality  ANY]
		x       [-end_station             ANY]
		x       [-include_sci             ANY]
		x       [-system_id               ANY]
		x       [-system_id_step          ANY]
		x       [-port_id                 ANY]
		x       [-rekey_behaviour         CHOICES dontRekey
		x                                 CHOICES rekeyContinuous
		x                                 CHOICES rekeyFixedCount]
		x       [-periodic_rekey_interval NUMERIC]
		x       [-periodic_rekey_attempts NUMERIC]
		x       [-rx_sak_pool_size        NUMERIC]
		x       [-dut_sci_mac             ANY]
		x       [-dut_sci_mac_step        ANY]
		x       [-dut_sci_port_id         ANY]
		x       [-dut_msb_of_xpn          ANY]
		x       [-override_tci_setting    ANY]
		x       [-version                 ANY]
		x       [-tci_es_bit              ANY]
		x       [-tci_sc_bit              ANY]
		x       [-tci_scb_bit             ANY]
		x       [-tci_e_bit               ANY]
		x       [-tci_c_bit               ANY]
		x       [-name                    ALPHA]
		x       [-vlan_id                 ANY]
		x       [-priority                ANY]
		x       [-tpid                    CHOICES ethertype8100
		x                                 CHOICES ethertype88a8
		x                                 CHOICES ethertype9100
		x                                 CHOICES ethertype9200
		x                                 CHOICES ethertype9300]
		x       [-rx_sak128               ANY]
		x       [-rx_sak256               ANY]
		x       [-rx_ssci                 ANY]
		x       [-rx_salt                 ANY]
		x       [-tx_sak128               ANY]
		x       [-tx_sak256               ANY]
		x       [-tx_ssci                 ANY]
		x       [-tx_salt                 ANY]
		
		 Arguments:
		    -mode
		        This option defines whether to Create/Modify/Delete/Enable/Disable Static Macsec.
		    -port_handle
		        The port on which the Static Macsec configuration will take place.
		    -handle
		        Specifies the parent node handle ( For ex : IP stack output of interface_config) on which Static Macsec is configured.
		        The Static Macsec handle(s) are returned by the procedure "emulation_static_macsec_config" when configuring Static Macsec on the Ixia interface.
		        For modify -mode, -handle can be Static Macsec's node handle.
		        For delete -mode, -handle is Static Macsec's node handle.
		        For enable/disable -mode, -handle can be Static Macsec's node handle or session(item) handle of these nodes.
		x   -enable_encrypted_vlan
		x       DEPRECATED: use enable_encrypted_vlans instead
		x   -encrypted_traffic_type
		x       Determines the set of MACsec functionalities supported by a Device Group. "Stateless L2-3" enables encryption of stateless L2-3 traffic at line rate, "Stateful L4-7" enables encryption and live decryption of stateful L4-7 traffic at lower throughput. Separate Device Groups need to be configured to get both the options on the same port.
		x   -source_ip
		x       IP address of the MACsec device. Used as the source IP of the MACsec traffic generated by the device.
		x   -source_ip_step
		x       Step argument of Source IP.
		x   -dut_mac
		x       MAC address of the connected interface of the DUT to be used as Gateway MAC for IP traffic.
		x   -dut_mac_step
		x       Step argument of DUT MAC.
		x   -send_grat_arp
		x       Determines whether Grat ARP should be sent out by each device with the configured MAC-IP mapping if encryption engine is "Hardware Based". If Grap ARP is disabled then static ARP entries need to be created at the DUT to avoid traffic loss.
		x   -fixed_pn
		x       Packet Number with which all MACsec packets will be sent out by all the devices of the Device Group.
		x   -mv_fixed_xpn
		x       8 bytes PN with which all packets will be sent out.
		x   -active_device
		x       Determines whether a MACsec device is active or not. If disabled, MACsec will not be started on the device.
		x   -cipher_suite
		x       The type of cipher suite. Options include: 1) GCM-AES-1282) GCM-AES-256 3) GCM-AES-XPN-1284) GCM-AES-XPN-256
		x   -confidentiality_offset
		x       Determines the confidentiality offset for both transmit and receive channel.
		x   -enable_encrypted_vlans
		x       Determines whether configured encrypted VLANs are included in MACsec packets or not. It can be configured per device.
		x   -encrypted_vlan_count
		x       Denotes the number of VLANs to be sent as encrypted VLANs. Maximum 6 VLANs can be configured as encrypted VLANs.
		x   -tx_sak_pool_size
		x       Determines the number of SAKs configured for each device for the Tx secure channel. Multiple SAKs are needed if Rekey scenario is to be simulated.
		x   -enable_confidentiality
		x       Determines whether transmitted MACsec payload is encrypted or not.
		x   -end_station
		x       Determines whether this device should act as the End Station.
		x   -include_sci
		x       Determines whether SCI should be included in the SecTAG of the transmitted MACsec traffic.
		x   -system_id
		x       System Identifier component of the SCI field with which MACsec traffic is transmitted.
		x   -system_id_step
		x       Step argument of System Id.
		x   -port_id
		x       Port ID component of SCI with which MACsec traffic is transmitted.
		x   -rekey_behaviour
		x       Determines the Rekey behavior.
		x   -periodic_rekey_interval
		x       Determines the time interval between two subsequent Rekey events. The timer starts with the first MACsec packet transmission from a device.
		x   -periodic_rekey_attempts
		x       Determines the number of times Rekey will happen after MACsec is started.
		x   -rx_sak_pool_size
		x       Determines the number of SAKs configured for each device for the Rx secure channel. Multiple SAKs are needed if DUT is expected to trigger Rekey during the test.
		x   -dut_sci_mac
		x       MAC component of SCI with which MACsec traffic is expected from the DUT.
		x   -dut_sci_mac_step
		x       Step argument of DUT SCI MAC.
		x   -dut_sci_port_id
		x       Port ID component of SCI with which MACsec traffic is expected from the DUT.
		x   -dut_msb_of_xpn
		x       The 32 most significant bits of the XPN that DUT will be using to construct the 64 bits XPN value when test starts.
		x   -override_tci_setting
		x       If enabled, the default TCI settings as per configuration will be allowed for overwriting. Used to simulate mal-configured SecTAG for negative testing.
		x   -version
		x       Version bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
		x   -tci_es_bit
		x       Determines the "ES" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
		x   -tci_sc_bit
		x       Determines the "SC" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
		x   -tci_scb_bit
		x       Determines the "SCB" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
		x   -tci_e_bit
		x       Determines the "E" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
		x   -tci_c_bit
		x       Determines the "C" bit in TCI field of SecTAG if "Override TCI Settings" is enabled.
		x   -name
		x       Name of NGPF element, guaranteed to be unique in Scenario
		x   -vlan_id
		x       12-bit VLAN ID in the VLAN tag.
		x   -priority
		x       3-bit user priority field in the VLAN tag.
		x   -tpid
		x       16-bit Tag Protocol Identifier (TPID) or EtherType in the VLAN tag.
		x   -rx_sak128
		x       128 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
		x   -rx_sak256
		x       256 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
		x   -rx_ssci
		x       4 bytes short SCI which DUT is expected to use for encrypting MACsec packets in case of XPN cipher suites.
		x   -rx_salt
		x       12 bytes Salt which DUT is expected to use for encrypting MACsec packets in case of XPN cipher suites.
		x   -tx_sak128
		x       128 bit value of Secure Association Key used for encryption of transmitted MACsec packets.
		x   -tx_sak256
		x       256 bit value of Secure Association Key with which DUT is expected to encrypt MACsec packets.
		x   -tx_ssci
		x       4 bytes short SCI used for encryption of transmitted MACsec packets in case of XPN cipher suites.
		x   -tx_salt
		x       12 bytes Salt used for encryption of transmitted MACsec packets in case of XPN cipher suites.
		
		 Return Values:
		    A list containing the ethernet protocol stack handles that were added by the command (if any).
		x   key:ethernet_handle       value:A list containing the ethernet protocol stack handles that were added by the command (if any).
		    $::SUCCESS | $::FAILURE
		    key:status                value:$::SUCCESS | $::FAILURE
		    When status is $::FAILURE, log shows the detailed information of failure.
		    key:log                   value:When status is $::FAILURE, log shows the detailed information of failure.
		    Handle of Static Macsec configured
		    key:static_macsec_handle  value:Handle of Static Macsec configured
		    Handle of Tx SAK Pool configured
		    key:tx_sak_pool_handle    value:Handle of Tx SAK Pool configured
		    Handle of Rx SAK Pool configured
		    key:rx_sak_pool_handle    value:Handle of Rx SAK Pool configured
		
		 Examples:
		
		 Sample Input:
		
		 Sample Output:
		
		 Notes:
		
		 See Also:
		
		'''
		hlpy_args = locals().copy()
		hlpy_args.update(kwargs)
		del hlpy_args['self']
		del hlpy_args['kwargs']

		not_implemented_params = []
		mandatory_params = []
		file_params = []

		try:
			return self.__execute_command(
				'emulation_static_macsec_config', 
				not_implemented_params, mandatory_params, file_params, 
				hlpy_args
			)
		except (IxiaError, ):
			e = sys.exc_info()[1]
			return make_hltapi_fail(e.message)
